#include "fusionwindow.h"
#include "ui_fusionwindow.h"

#include <QKeyEvent>

#include <src/Reconstruction/CloudFromDepth.h>

#include "../Cams/Freenect2/Freenect2Cam.h"
#include "../Cams/Freenect2/Freenect2CamManager.h"
#include "../Cams/OpenCVCam/OpenCVCam.h"
#include "../Reconstruction/CloudFromDepth.h"
#include "../Tracker/CoordSystemUnifier.h"

#include "../ReactDomain.h"


FusionWindow::FusionWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::FusionWindow)
{
    ui->setupUi(this);

    /*if (cam->Open()) {
        auto capabilities = cam->getStreamCapabilities();

        if (capabilities.find(StreamTypes::DEPTH) != capabilities.end()) {
            auto stretched = Transform(cam->ImageStreams[StreamTypes::DEPTH], [](cv::Mat image) {
                // Histogram Equalization
                double min;
                double max;
                cv::minMaxIdx(image, &min, &max);
                cv::Mat adjMap = cv::Mat();
                // expand your range to 0..255. Similar to histEq();
                //(*image).convertTo(adjMap, CV_8UC1, 255 / (max-min), -255*min/(max-min)); // try this if the other method fails
                image.convertTo(adjMap, CV_8UC1, 255 / (max - min), -min);
                return adjMap;
            });
            m_depthObserver = Observe(stretched, [this](cv::Mat image) {
                ui->openCVviewer1->showImage(image);
            });

            // point cloud
            m_cloudObserver = Observe(
                    m_pointsMapper.PointPosEvents,
                    [this](const cv::Mat &points) {
                        ui->pointCloudViewer->setPointPos(points);
                    });
        }

        if (capabilities.find(StreamTypes::IR) != capabilities.end()) {
            auto stretched = Transform(cam->ImageStreams[StreamTypes::IR], [](cv::Mat image) {
                // Histogram Equalization
                double min;
                double max;
                cv::minMaxIdx(image, &min, &max);
                cv::Mat adjMap = cv::Mat();
                // expand your range to 0..255. Similar to histEq();
                //(*image).convertTo(adjMap, CV_8UC1, 255 / (max-min), -255*min/(max-min)); // try this if the other method fails
                image.convertTo(adjMap, CV_8UC1, 255 / (max - min), -min);
                return adjMap;
            });
            m_irObserver = Observe(stretched, [this](cv::Mat image) {
                ui->openCVviewer2->showImage(image);
            });
        }

        if (capabilities.find(StreamTypes::COLOR) != capabilities.end()) {
            m_colorObserver = Observe(cam->ImageStreams[StreamTypes::COLOR], [this](cv::Mat image) {
                ui->openCVviewer3->showImage(image);
            });

            // point cloud colors
            m_cloudObserver = Observe(
                    m_pointsMapper.PointColorsEvents,
                    [this](const cv::Mat &colors) {
                        ui->pointCloudViewer->setPointColors(colors);
                    });
        }
    }*/
}

bool FusionWindow::init()
{
    bool record = false;

    std::string recordingPrefix = "colorTest";

    // only use opencl if libfreenect2 was compiled with it
#ifdef LIBFREENECT2_WITH_OPENCL_SUPPORT
    m_cam = std::make_shared<Freenect2Cam>("", new libfreenect2::OpenCLPacketPipeline());
#else
    m_cam = std::make_shared<freenect2cam::Freenect2Cam>();
#endif

    if( record )
    {
        // create, open and track the pattern in camera 1
        if( !m_cam->Open() )
            return false;

        m_recorder = std::make_shared<FFmpegCamRecorder>();
        if( !m_recorder->start( recordingPrefix, m_cam->Streams, 30, 0, true ) )
            return false;

        auto stretched = Transform(m_cam->Streams.GetStreamFirst<ImageStream>(StreamTypes::IR)->events, [](cv::Mat image) {
            // Histogram Equalization
            double min;
            double max;
            cv::minMaxIdx(image, &min, &max);
            cv::Mat adjMap = cv::Mat();
            // expand your range to 0..255. Similar to histEq();
            //(*image).convertTo(adjMap, CV_8UC1, 255 / (max-min), -255*min/(max-min)); // try this if the other method fails
            image.convertTo(adjMap, CV_8UC1, 255 / (max - min), -min);
            return adjMap;
        });
        m_irObserver1 = Observe(stretched, [this](cv::Mat image) {
            ui->openCVviewer1->showImage(image);
        });
    }
    else
    {
        // open the recorded data
        m_player = std::make_shared<FFmpegCamPlayer>(recordingPrefix);
        m_player->Open();

        // open the kinect
        if( !m_cam->Open() )
            return false;

        // create the trackers
        m_patternTracker1 = std::make_shared<PatternTracker>( m_player->Streams, cv::Size(5,15), 0.02 );
        m_patternTracker1->start();

        m_patternTracker2 = std::make_shared<PatternTracker>( m_cam->Streams, cv::Size(5,15), 0.02 );
        m_patternTracker2->start();

        m_trackerObserver1 = Observe( m_patternTracker1->m_coordSystemStream, [this](const cv::Matx44d mat)
        {
            if (!m_mat1_found) {
                m_patternMat1 = mat.inv();
                m_mat1_found = true;
            }
        });
        m_trackerObserver2 = Observe( m_patternTracker2->m_coordSystemStream, [this](const cv::Matx44d mat)
        {
            m_patternMat2 = mat.inv();
        });

//        auto unifier = std::make_shared<CoordSystemUnifier>();
//        int tracker1ID = unifier->addTracker( patternTracker1 );
//        int tracker2ID = unifier->addTracker( patternTracker2 );

//        unifier->start();

        m_pointsMapper1 = std::make_shared<CloudFromDepth>( m_player->Streams );
        m_pointsMapper2 = std::make_shared<CloudFromDepth>( m_cam->Streams );

        m_cloudObserver1 = Observe( m_pointsMapper1->PointPosEvents, [this](const cv::Mat mat)
        {
            cv::Mat out;
            cv::perspectiveTransform( mat.reshape(3), out, m_patternMat1 );
            m_pointCloud1 = out.reshape(1);
            cv::Mat submat = m_pointCloud(cv::Rect(0,0,3,512 * 424));
            m_pointCloud1.copyTo(submat);
            ui->pointCloudViewer->setPointPos( m_pointCloud );
//
//            if( !m_pointCloud2.empty() && m_cloudMutex.try_lock() )
//            {
//                m_pointCloud1.push_back( m_pointCloud2 );
//                ui->pointCloudViewer->setPointPos( m_pointCloud1 );
//
//                m_pointCloud1 = cv::Mat();
//                m_pointCloud2 = cv::Mat();
//                m_cloudMutex.unlock();
//            }
        });
        m_cloudObserver2 = Observe( m_pointsMapper2->PointPosEvents, [this](const cv::Mat mat)
        {
            cv::Mat out;
            cv::perspectiveTransform( mat.reshape(3), out, m_patternMat2 );
            m_pointCloud2 = out.reshape(1);
            // copy to second part of global cloud
            cv::Mat submat = m_pointCloud(cv::Rect(0, 512 * 424, 3, 512 * 424));
            m_pointCloud2.copyTo(submat);
            ui->pointCloudViewer->setPointPos( m_pointCloud ); //TODO correct?

//            if( !m_pointCloud1.empty() && m_cloudMutex.try_lock() )
//            {
//                m_pointCloud1.push_back( m_pointCloud2 );
//                ui->pointCloudViewer->setPointPos( m_pointCloud1 );
//
//                m_pointCloud1 = cv::Mat();
//                m_pointCloud2 = cv::Mat();
//                m_cloudMutex.unlock();
//            }
        });

        m_colorObserver1 = Observe( m_pointsMapper1->PointColorsEvents, [this](const cv::Mat colors)
        {
            m_pointColors1 = colors.reshape(0, 512 * 424);
            cv::Mat submat = m_pointColors(cv::Rect(0, 0, 1, 512 * 424));
            m_pointColors1.copyTo(submat);
            ui->pointCloudViewer->setPointColors( m_pointColors );

//            if( !m_pointColors2.empty() && m_colorMutex.try_lock() )
//            {
//                m_pointColors1.push_back( m_pointColors2 );
//                ui->pointCloudViewer->setPointColors( m_pointColors1 );
//
//                m_pointColors1 = cv::Mat();
//                m_pointColors2 = cv::Mat();
//                m_colorMutex.unlock();
//            }
        });

        m_colorObserver2 = Observe( m_pointsMapper2->PointColorsEvents, [this](const cv::Mat colors)
        {
            m_pointColors2 = colors.reshape(0, 512 * 424);
            cv::Mat submat = m_pointColors(cv::Rect(0, 512 * 424, 1, 512 * 424));
            m_pointColors2.copyTo(submat);
            ui->pointCloudViewer->setPointColors( m_pointColors );

//            if( !m_pointColors1.empty() && m_colorMutex.try_lock() )
//            {
//                m_pointColors1.push_back( m_pointColors2 );
//                ui->pointCloudViewer->setPointColors( m_pointColors1 );
//
//                m_pointColors1 = cv::Mat();
//                m_pointColors2 = cv::Mat();
//                m_colorMutex.unlock();
//            }
        });
    }

    return true;
}

FusionWindow::~FusionWindow() {
    if( m_recorder ) m_recorder->stop();
    if( m_cam ) m_cam->Close();
    if( m_player ) m_player->Close();

    delete ui;
}

void FusionWindow::keyPressEvent(QKeyEvent* event)
{
    // make the app quit on Ctrl+Q or ESC
    if( ( event->key() == Qt::Key_Q && event->modifiers() == Qt::ControlModifier ) || event->key() == Qt::Key_Escape )
        close();

    QMainWindow::keyPressEvent(event);
}
