
#ifndef DYNCAM_FFMPEGRECORDER_H
#define DYNCAM_FFMPEGRECORDER_H

#include "../../ReactDomain.h"

#include <vector>
#include <string>
#include <chrono>
#include "src/Core/IntrinsicsStream.h"

#include <opencv2/opencv.hpp>

extern "C" {
#include <libavutil/opt.h>
#include <libavcodec/avcodec.h>
#include <libavutil/channel_layout.h>
#include <libavutil/common.h>
#include <libavutil/imgutils.h>
#include <libavutil/mathematics.h>
#include <libavutil/samplefmt.h>
#include <libswscale/swscale.h>
}

#include "../BaseCam.h"

namespace dyncam {

class FFmpegStreamRecorder
{
public:
    FFmpegStreamRecorder() = default;
    ~FFmpegStreamRecorder();

    bool start(std::string filename, cv::Size frameSize, int openCVType, ushort maxFPS, EventsT<cv::Mat> input, 
                     int compress = 0, bool hardwareAcc = false );
    void stop();

private:
    AVCodec* m_codec = nullptr;
    AVCodecContext* m_codecContext = nullptr;

    AVFrame* m_frame = nullptr;
    AVPacket m_packet;

    SwsContext* m_swCtx = nullptr;
    unsigned int m_frameNumber = 0;
    std::chrono::time_point<std::chrono::high_resolution_clock> m_startedRecording;

    std::ofstream m_fileOutput;

    ObserverT m_streamObserver;
};

class FFmpegCamRecorder
{
public:
    FFmpegCamRecorder() = default;
    ~FFmpegCamRecorder() = default;

    bool start(const std::string &filePrefix, StreamCollection &streams,
               ushort maxFPS, int compress = 0, bool hardwareAcc = false );
    void stop();

private:
    std::list<FFmpegStreamRecorder> m_streamRecorders;
};

} // namespace dyncam
#endif // DYNCAM_FFMPEGRECORDER_H
