//
// Created by ascadian on 4/2/19.
//

#include "IntrinsicsStreamSession.h"
#include "src/Core/IntrinsicsStream.h"
#include "RequestStreams.h"
#include "Connection.h"

namespace dyncam {

IntrinsicsStreamSession::IntrinsicsStreamSession(boost::asio::ip::basic_endpoint<boost::asio::ip::tcp> endpoint,
                                                 std::shared_ptr<IntrinsicsStream> stream) :
    m_stream(std::move(stream)){
    m_endpoint = endpoint;
    start(endpoint);
}

void IntrinsicsStreamSession::sendStreamRequest() {
    std::cout << "Client: Requesting intrinsics stream" << std::endl;

    // bind connection info in lambda to make sure it stays in scope during async write
    std::shared_ptr<RequestStreams> reqStreamList = std::make_shared<RequestStreams>(m_stream->GetId(), m_stream->getType());
    m_connection->async_write(*reqStreamList,
                              [this, reqStreamList=reqStreamList] (const boost::system::error_code &e) {
                                  if (e) {
                                      std::cerr << "Client: " << e.message() << std::endl;
                                  } else {
                                      std::cout << "Client: RequestStreams send" << std::endl;
                                      readStream();
                                  }
                              });
}

void IntrinsicsStreamSession::readStream() {
    auto m = std::make_shared<CamIntrinsics>();
    m_connection->async_read(*m,
        [this, m](const boost::system::error_code &e) mutable {
         if (e) {
             std::cerr << "ClientSess: " << e.message() << std::endl;
         } else {
             m_stream->Intrinsics <<= *m;
             std::cout << "ClientSess received intrinsics: " << std::endl << m << std::endl;
             readStream();
         }
        });
}

void IntrinsicsStreamSession::on_connected() {
    ClientBase::on_connected();
    sendStreamRequest();
}

} // namespace dyncam {