//
// Created by ascadian on 9/21/18.
//

#pragma once
#include <src/Core/Stream.h>
#include "src/Compression/BaseCompression.h"

#include "src/Core/Serialization.h"

namespace dyncam {

/*
 * Request either to start one or more streams, or to get a StreamCollection with
 * available streams.
 *
 * If startStreams contains no elements, this counts a StreamCollection request.
 */
class RequestStreams {

public:
    RequestStreams() = default;
    explicit RequestStreams(int streamID, const StreamTypes streamType);
    RequestStreams(int streamID, const StreamTypes streamType, std::shared_ptr<BaseCompression> compression);

    bool isCollectionRequest();
    std::shared_ptr<BaseCompression> GetCompression();
    int getStreamID();
    StreamTypes getType();


    template<class Archive>
    void serialize(Archive &ar) {
        ar(m_requestCollection, m_streamID, m_compression, m_streamType);
    }

    friend std::ostream & operator<<(std::ostream & stream, RequestStreams const & v) {
        std::stringstream os;
        cereal::JSONOutputArchive archive_out(os);
        archive_out(v);
        stream << os.str() << std::endl;
        return stream;
    }

protected:
    bool m_requestCollection = true;
    int m_streamID = -1;
    std::shared_ptr<BaseCompression> m_compression;
    StreamTypes m_streamType = StreamTypes::UNKNOWN;
};

} // namespace dyncam
