#include <utility>

//
// Created by ascadian on 9/24/18.
//

#include "StreamClient.h"
#include "RequestStreams.h"
#include "Connection.h"
#include "src/Core/StreamCollection.h"
#include "src/Core/ImageStream.h"
#include "src/Core/IntrinsicsStream.h"
#include "src/Cams/NetCam/ImageStreamSession.h"
#include "src/Cams/NetCam/IntrinsicsStreamSession.h"
#include "CompressedStream.h"
#include "CompressedStreamSession.h"

namespace dyncam {

StreamClient::StreamClient() : ClientBase(),
                               m_streams(std::make_unique<StreamCollection>()) {

}

void StreamClient::on_connected() {
    ClientBase::on_connected();
    sendListRequest();
}

void StreamClient::sendListRequest() {
    std::cout << "Client: Requesting streamList" << std::endl;

    // bind connection info in lambda to make sure it stays in scope during async write
    std::shared_ptr<RequestStreams> reqStreamList = std::make_shared<RequestStreams>();
    m_connection->async_write(*reqStreamList,
                              [this, reqStreamList=reqStreamList] (const boost::system::error_code &e) {
                                  if (e) {
                                      std::cerr << "Client: " << e.message() << std::endl;
                                  } else {
                                      std::cout << "Client: RequestStreams send" << std::endl;
                                      readStreamColl();
                                  }
                              });
}

void StreamClient::readStreamColl() {
    std::cout << "Client: Reading StreamColl" << std::endl;

    m_connection->async_read(*m_streams,
                             [this](const boost::system::error_code &e) {
                                 if (e) {
                                     std::cerr << "Client: " << e.message() << std::endl;
                                 } else {
                                     std::cout << "Client received: " << std::endl << *m_streams << std::endl;
                                 }
                             });
}

void StreamClient::requestStream(std::shared_ptr<ImageStream> stream) {
    std::cout << "Client: Requesting ImageStream" << std::endl;

    // make sure we know the stream
    assert(m_streams->GetStreamById<ImageStream>(stream->GetId()));

    m_streamSessions.push_front(std::make_unique<ImageStreamSession>(m_endpoint, std::move(stream)));
}

void StreamClient::requestStream(std::shared_ptr<IntrinsicsStream> stream) {
    std::cout << "Client: Requesting IntrinsicsStream" << std::endl;

    // make sure we know the stream
    assert(m_streams->GetStreamById<IntrinsicsStream>(stream->GetId()));

    m_streamSessions.push_front(std::make_unique<IntrinsicsStreamSession>(m_endpoint, std::move(stream)));
}


void StreamClient::requestStream(std::shared_ptr<dyncam::CompressedStream> stream) {
    std::cout << "Client: Requesting CompressedStream" << std::endl;

    m_streamSessions.push_front(std::make_unique<CompressedStreamSession>(m_endpoint, std::move(stream)));
}

} // namespace dyncam
