//
// Created by ascadian on 12/1/17.
//

#include "StreamServer.h"
#include "ServerSession.h"
#include <thread>
#include <boost/asio.hpp>
#include <boost/bind.hpp>

namespace dyncam {

using boost::asio::ip::tcp;

StreamServer::StreamServer(const unsigned short port) :
    m_port(port),
    m_endpoint(tcp::v4(), m_port),
    m_acceptor(m_io_service, m_endpoint) {
    m_thread = std::thread(&StreamServer::threadMain, this);
}

StreamServer::~StreamServer() {
    shutdown();
}

void StreamServer::shutdown() {
    try {
        m_io_service.stop();
        m_thread.join();
    } catch (std::exception &e) {
        std::cerr << "Serv: Shutdown exception: " << e.what() << "\n";
    }
}

void StreamServer::threadMain() {
    try {
        std::cout << "Serv: Starting server" << std::endl;
        acceptConnections();
        boost::asio::io_service::work idleWork(m_io_service);
        m_io_service.run();
        std::cout << "Serv: work done" << std::endl;
    }
    catch (std::exception &e) {
        std::cerr << "Serv: Mainthread exception: " << e.what() << "\n";
    }
}

void StreamServer::acceptConnections() {
    std::cout << "Serv: Accepting..." << std::endl;
    auto session = std::make_shared<ServerSession>(m_io_service, this);
    m_acceptor.async_accept(session->socket(),
                            [this, session](boost::system::error_code ec) {
                                std::cout << "Serv: New connection..." << std::endl;
                                if (!ec) {
                                    session->start();
                                }
                                // accept further connection
                                acceptConnections();
                            });
}
void StreamServer::AddStream(std::shared_ptr<Stream> s) {
    m_streams->AddStream(s);
}

} // namespace dyncam