//
// Created by ascadian on 11/30/17.
//

#include "QuantizedCompression.h"

namespace dyncam {


EventsT<cv::Mat> QuantizedCompression::compress(const EventsT<cv::Mat> &inStream) {
    auto trans = Transform(inStream, [] (const cv::Mat &image) {
        // we also convert from float to int and currently only back to CV_32F
        assert(image.depth() == CV_32F);

		cv::Mat fullrange = image*8.191875; //65536 (max value 16 bit int) / 8000 (max dist in cm of kinect v2) = 8.191875
        // replace NaN by 0
		fullrange.setTo(0, fullrange != fullrange);

        // TODO check if any value is smaller or larger than min(16S1)..max(16S1)
        // check that nothing is too big or too small
//        auto tooSmallCount = cv::countNonZero(meters < 0);
//        if (tooSmallCount != 0) {
//            std::cout << meters << std::endl;
//        }
//        assert(tooSmallCount == 0);
        //assert(cv::countNonZero(meters > maxDist) == 0);
        cv::Mat target;
        switch (image.channels()) {
            case 1: {
				fullrange.convertTo(target, CV_16UC1);
                break;
            }
            case 3: {
				fullrange.convertTo(target, CV_16UC3);
                break;
            }
			case 4: {
				fullrange.convertTo(target, CV_16UC4);
				break;
			}
            default:
                std::cerr << "Mat not supported by QuantizedCompression: channels" << image.channels() << std::endl;
                break;
        }
        return target;
    });
    D::SetNodeName(*trans.ptr_, "QuantizedCompression::compress");
    return trans;
}

EventsT<cv::Mat> QuantizedCompression::deCompress(const EventsT<cv::Mat> &inStream) {
    auto trans = Transform(inStream, [] (const cv::Mat &image) {
        assert(image.depth() == CV_16S);

        cv::Mat target;
        switch (image.channels()) {
            case 1: {
                image.convertTo(target, CV_32FC1);
                break;
            }
            case 3: {
                image.convertTo(target, CV_32FC3);
                break;
            }
			case 4: {
				image.convertTo(target, CV_32FC4);
				break;
			}
            default:
                std::cerr << "Mat not supported by QuantizedCompression: channels" << image.channels() << std::endl;
                break;
        }

		cv::Mat originValues = target / 8.191875;
        return originValues;
    });
    D::SetNodeName(*trans.ptr_, "QuantizedCompression::deCompress");
    return trans;
}


QuantizedCompression::QuantizedCompression() : BaseCompression() {

}

std::string QuantizedCompression::ToString() {
    return "QuantizedCompression";
}

} // namespace dyncam