//
// Created by ascadian on 6/11/18.
//

#pragma once

#include <memory>
#include "src/ReactDomain.h"
#include "Stream.h"

namespace dyncam {

class StreamCollection {

public:
    StreamCollection();

    void AddStream(std::shared_ptr<Stream> s);

    const std::vector<std::shared_ptr<Stream>> GetAllStreams() const;

    /**
     *
     * @tparam T
     * @param streamType
     * @return nullptr if not found
     */
    template <class T>
    std::shared_ptr<T> GetStreamFirst(StreamTypes streamType) {
        static_assert(std::is_base_of<Stream, T>::value, "T must derive from Stream");

        auto result = std::find_if(streams_.begin(), streams_.end(),
                                   [&](const std::shared_ptr<Stream> &s){
            return s->stream_type_ == streamType;
        });
        if (streams_.end() != result) {
            return std::dynamic_pointer_cast<T>(*result);
        }
        return nullptr;
    }

    /**
     *
     * @tparam T
     * @param streamType
     * @param parentId
     * @return nullptr if not found
     */
    template <class T>
    std::shared_ptr<T> GetStreamFirst(StreamTypes streamType, int parentId) {
        static_assert(std::is_base_of<Stream, T>::value, "T must derive from Stream");

        auto result = std::find_if(streams_.begin(), streams_.end(),
                                   [&](const std::shared_ptr<Stream> &s){
                                       return (s->stream_type_ == streamType)
                                           && s->parent_id_ == parentId;
                                   });
        if (streams_.end() != result) {
            return std::dynamic_pointer_cast<T>(*result);
        }
        return nullptr;
    }

    /**
     *
     * @tparam T
     * @param streamType
     * @param parentId
     * @return nullptr if not found
     */
    template <class T>
    std::shared_ptr<T> GetStreamById(int streamId) {
        static_assert(std::is_base_of<Stream, T>::value, "T must derive from Stream");

        auto result = std::find_if(streams_.begin(), streams_.end(),
                                   [&](const std::shared_ptr<Stream> &s){
                                       return s->id_ == streamId;
                                   });
        if (streams_.end() != result) {
            return std::dynamic_pointer_cast<T>(*result);
        }
        return nullptr;
    }

    const std::string GetSerial() const;
    void SetSerial(const std::string serial);

    template <class Archive>
    void serialize(Archive & ar)
    {
        ar(serial_, streams_);
    }

    friend std::ostream & operator<<(std::ostream & stream, StreamCollection const & v) {
        std::stringstream os;
        cereal::JSONOutputArchive archive_out(os);
        archive_out(v);
        stream << os.str() << std::endl;
        return stream;
    }

protected:
    std::vector<std::shared_ptr<Stream>> streams_;

    // serial number of camera if available
    std::string serial_;
};

} // namespace dyncam
