#include <opencv2/opencv.hpp>

#include <chrono>
#include <functional>
#include <memory>
#include <random>
#include <thread>
#include <lz4.h>
#include "Cams/Freenect2/Freenect2Cam.h"
#include "Cams/Freenect2/Freenect2CamManager.h"

#include "ReactDomain.h"


void freenectTest(){
    std::cout << "Testing C++React with freenect2\n\n";
}



int main(int argc,char *argv[])
{
    using namespace dyncam;
	
    auto cam = std::make_unique<dyncam::Freenect2Cam>();
    cam->Open();

    auto depth = cam->Streams.GetStreamFirst<ImageStream>(StreamTypes::DEPTH);
	float TRZ = 0.8f;  // higher value for more agressive filtering of holes
	float TRNZ = 0.4f; // lower value for more agressive filtering of outliers
	int motionTR = 500;
	int radius = 2;
	cv::Mat n1=cv::Mat(424, 512, CV_8UC1);
	cv::Mat n2=cv::Mat(424, 512, CV_8UC1);
	cv::Mat n3=cv::Mat(424, 512, CV_8UC1);
	n1.setTo(0);
	n2.setTo(0);
	n3.setTo(0);
    EventsT<cv::Mat> stretched;
	EventsT<cv::Mat> denoised;
	EventsT<cv::Mat> denoisedT;
    EventsT<cv::Mat> colored;
	EventsT<cv::Mat> normals;
    Observer<D> depthObs;
	Observer<D> denoiseObs;
	Observer<D> denoiseObsT;
    Observer<D> colorDepthObs;
	Observer<D> normalObs;
    if (depth) {
        stretched = Transform(depth->events, [depth](const cv::Mat image) {

					// Histogram Equalization
					double min;
					double max;
					cv::minMaxIdx(image, &min, &max);
					cv::Mat adjMap = cv::Mat();
					// expand your range to 0..255. Similar to histEq();
					//(*image).convertTo(adjMap, CV_8UC1, 255 / (max-min), -255*min/(max-min)); // try this if the other method fails
					image.convertTo(adjMap, CV_8UC1, 255 / (max - min), -min);

                    return adjMap;
                });

        colored = Transform(stretched, [](const cv::Mat image) {
                    // Convert grey scaled image to tone mapped
                    cv::Mat falseColorsMap = cv::Mat();
                    applyColorMap(image, falseColorsMap, cv::COLORMAP_AUTUMN);
                    return falseColorsMap;
                });

		/*denoised = Transform(stretched, [TRZ,TRNZ,radius](const cv::Mat image) {
			cv::Mat filteredMat = image.clone();
			
			int rows = image.rows;
			int cols = image.cols;
			//std::vector<std::pair<int, int>> Histogramm;//value,count

			cv::parallel_for_(cv::Range(0, rows), [&filteredMat, rows, cols, radius, &image, TRZ, TRNZ](const cv::Range& range) {
				for (int r = range.start; r < range.end; r++)
				{
					int i = r;
					std::vector<std::pair<int, int>> Histogramm;//value,count
					for (int j = 0; j < cols;++j)
					{
						for (int x = -radius; x <= radius; ++x)
						{
							for (int y = -radius; y <= radius; ++y)
							{
								if (x != 0 && y != 0)
								{
									bool exists = false;
									for (int n = 0; n < Histogramm.size();++n)
									{
										if (Histogramm[n].first == image.at<UINT8>(i + x, j + y))
										{
											Histogramm[n].second++;
											exists = true;
											break;
										}

									}
									if (!exists)
									{
										Histogramm.push_back(std::pair<int, int>(image.at<UINT8>(i + x, j + y), 1));
									}
								}

							}
						}

						//count zero-pixels in neighborhood
						int NumZeroes = 0;
						for (auto Pair : Histogramm)
						{
							if (Pair.first == 0)
							{
								NumZeroes += Pair.second;
								break;
							}
						}

						//check if filtering is needed for holes
						if (image.at<UINT8>(i, j) == 0 && (NumZeroes / 24.0f) < TRZ)
						{
							int Value = 0;
							int MaxCount = 0;
							for (auto Pair : Histogramm)
							{
								if (Pair.second > MaxCount)
								{
									MaxCount = Pair.second;
									Value = Pair.first;
								}
							}
							filteredMat.at<UINT8>(i, j) = Value;
						}//check if filtering is needed for outlier
						else if (image.at<UINT8>(i, j) != 0 && (NumZeroes / 24.0f) > TRNZ)
						{
							filteredMat.at<UINT8>(i, j) = 0;
						}
						Histogramm.clear();
					}
				}
			});

			return filteredMat;
		});

		denoisedT = Transform(denoised, [&n1, &n2, &n3, motionTR](const cv::Mat image) {
			cv::Mat filteredMat = image.clone();

			int rows = image.rows;
			int cols = image.cols;

			for (int i = 0; i < rows; ++i)
			{
				for (int j = 0; j < cols; ++j)
				{
					if (abs(image.at<UINT8>(i,j) - n1.at<UINT8>(i,j)) < motionTR)
					{
						filteredMat.at<UINT8>(i, j) = (n1.at<UINT8>(i, j) * 0.25 + n2.at<UINT8>(i, j) * 0.25 + n3.at<UINT8>(i, j) * 0.25 + image.at<UINT8>(i, j) *0.25);
					}
				}
			}

			n3 = n2;
			n2 = n1;
			n1 = image;
			/*auto start = std::chrono::system_clock::now();
			auto end = std::chrono::system_clock::now();
			auto diff = end - start;
			std::cout << "duration: " << std::chrono::duration_cast<std::chrono::milliseconds>(diff).count() << std::endl;*/
			//return filteredMat;
		//});

		/*normals = Transform(stretched, [](const cv::Mat image)
		{
			cv::Mat normalMap  = cv::Mat(image.size(), CV_32FC3);
			int rows = image.rows;
			int cols = image.cols;

			for (int i = 0; i < rows; ++i)
			{
				for (int j = 0; j < cols; ++j)
				{
					float dzdx = (image.at<UINT8>(i + 1, j) - image.at<UINT8>(i - 1, j)) / 2.0f;
					float dzdy = (image.at<UINT8>(i, j + 1) - image.at<UINT8>(i, j - 1)) / 2.0f;

					cv::Vec3f d(-dzdx, -dzdy, 0.010f);
					cv::Vec3f n = normalize(d);

					normalMap.at<cv::Vec3f>(i, j) = n;
				}
			}
			return normalMap;
		});*/


        // create window first as the callback is multithreaded and leads to opencv problems
        cv::namedWindow("Depth", cv::WINDOW_NORMAL);
        depthObs = Observe(stretched, [](const cv::Mat someImage) {
                    cv::imshow("Depth", someImage);
                    //cv::waitKey(1);
                });

        cv::namedWindow("Colored depth", cv::WINDOW_NORMAL);
        colorDepthObs = Observe(colored, [](const cv::Mat someImage) {
                    cv::imshow("Colored depth", someImage);
                    //cv::waitKey(1);
                });

		/*cv::namedWindow("Normals", cv::WINDOW_NORMAL);
		normalObs = Observe(normals, [](const cv::Mat someImage) {
			cv::imshow("Normals", someImage);
		});*/

		/*cv::namedWindow("Denoised", cv::WINDOW_NORMAL);
		denoiseObs = Observe(denoised, [](const cv::Mat someImage) {
			cv::imshow("Denoised", someImage);
		});

		cv::namedWindow("DenoisedT", cv::WINDOW_NORMAL);
		denoiseObsT = Observe(denoisedT, [](const cv::Mat someImage) {
			cv::imshow("DenoisedT", someImage);
		});*/
    }

    auto color = cam->Streams.GetStreamFirst<ImageStream>(StreamTypes::COLOR);
    Observer<D> colorObs;
    if (color) {
        cv::namedWindow("Color", cv::WINDOW_NORMAL);
        colorObs = Observe(color->events, [](const cv::Mat someImage) {
            cv::imshow("Color", someImage);
        });
    }


    bool stop = false;
    while (!stop)
        stop = (cv::waitKey(10) != -1);

    return EXIT_SUCCESS;
}
