//
// Created by ascadian on 12/4/17.
//

#include "TestUtils.h"
#include "gtest/gtest.h"
#include "Barrier.h"

int TestUtils::compareMats(const cv::Mat &img1, const cv::Mat &img2, const bool expectEqual) {
    return countNonZero(img1 != img2);
}

void TestUtils::compareFirstImageTwoStreams(const EventsT<cv::Mat> &a, const EventsT<cv::Mat> &b) {
    Barrier barrier;
    auto observer = compareFirstImageTwoStreams(a, b, barrier);
    barrier.count_down_and_wait();
}

Observer<D> TestUtils::compareFirstImageTwoStreams(const EventsT<cv::Mat> &a, const EventsT<cv::Mat> &b, Barrier &barrier) {
    EXPECT_TRUE(barrier.count_up()) << "Could not increase the lock counter";

    EventsT<std::tuple<cv::Mat, cv::Mat>> joined = Join(a, b);
    Observer<D> obs = Observe(joined, [&](const std::tuple<cv::Mat, cv::Mat> &images) {
        cv::Mat img1 = std::get<0>(images);
        cv::Mat img2 = std::get<1>(images);

        auto diff = TestUtils::compareMats(img1, img2);
        EXPECT_TRUE(diff == 0) << "Images difference unexpected, diff: " << cv::countNonZero(diff);
        barrier.count_down();
        return ObserverAction::stop_and_detach;
    });
    return obs;
}
