//
// Created by ascadian on 12/1/17.
//

#include <memory>
#include "gtest/gtest.h"
#include "../Cams/TestCam/TestCam.h"
#include "src/ReactDomain.h"
#include "src/Cams/NetCam/StreamServer.h"
#include "TestUtils.h"
#include <opencv2/opencv.hpp>
#include <src/Cams/NetCam/StreamClient.h>
#include "Barrier.h"

namespace {

class NetCamTest : public ::testing::Test {
protected:

    NetCamTest() {
    }

    virtual ~NetCamTest() {
        // You can do clean-up work that doesn't throw exceptions here.
    }
};

TEST_F(NetCamTest, ConnectionClassTest) {
    std::shared_ptr<TestCam> cam = std::make_shared<TestCam>();

    // server with one stream
    auto server = std::make_shared<StreamServer>();
    const auto depthStream = cam->Streams.GetStreamFirst<ImageStream>(StreamTypes::DEPTH);
    const auto intrStream = cam->Streams.GetStreamFirst<IntrinsicsStream>(StreamTypes::INTRINSICS);
    server->AddStream(depthStream);
    server->AddStream(intrStream);

    std::this_thread::sleep_for(std::chrono::milliseconds(10));

    // client automatically requests stream list from server
    StreamClient client;
    client.start("127.0.0.1");
    std::this_thread::sleep_for(std::chrono::milliseconds(100));
    EXPECT_EQ(client.m_streams->GetAllStreams().size(), 2);

    // request to stream all streams
    client.requestStream(client.m_streams->GetStreamFirst<ImageStream>(StreamTypes::DEPTH));
    client.requestStream(client.m_streams->GetStreamFirst<IntrinsicsStream>(StreamTypes::INTRINSICS));

    // currently we overwrite the events in the stream collection as soon as we request the stream,
    // so observe after request
    std::this_thread::sleep_for(std::chrono::milliseconds(10));

    // add a second image stream so the second client gets 2 image streams
    const auto &colorStream = cam->Streams.GetStreamFirst<ImageStream>(StreamTypes::COLOR);
    server->AddStream(colorStream);

    StreamClient client2;
    client2.start("127.0.0.1");
    std::this_thread::sleep_for(std::chrono::milliseconds(100));
    EXPECT_EQ(client2.m_streams->GetAllStreams().size(), 3);
    // Note: the first client does not know about the color stream
    client2.requestStream(client2.m_streams->GetStreamFirst<ImageStream>(StreamTypes::COLOR));


    const auto depthStream_client = client.m_streams->GetStreamFirst<ImageStream>(StreamTypes::DEPTH);
    const auto intrStream_client = client.m_streams->GetStreamFirst<IntrinsicsStream>(StreamTypes::INTRINSICS);
    const auto colorStream_client2 = client2.m_streams->GetStreamFirst<ImageStream>(StreamTypes::COLOR);

    Barrier barrier;
    // keep references to the observers here
    auto o1 = TestUtils::compareFirstImageTwoStreams(depthStream->events, depthStream_client->events, barrier);
    auto o2 = TestUtils::compareFirstImageTwoStreams(colorStream->events, colorStream_client2->events, barrier);

    std::cout << "Starting cam" << std::endl;
    cam->Open();
    barrier.count_down_and_wait();

    EXPECT_EQ(intrStream->Intrinsics(), intrStream_client->Intrinsics());

    cam->Close();

    // change intrinsics on the server
    auto newIntrinsics = CamIntrinsics(1, 2, 3, 4, 5, 6, 7, 8, 9);
    intrStream->Intrinsics.Set(newIntrinsics);

    // the client should have the new intrinsics now
    std::this_thread::sleep_for(std::chrono::milliseconds(10));
    EXPECT_EQ(newIntrinsics, intrStream_client->Intrinsics());

    std::cout << "Test end" << std::endl;
}

}  // namespace
