//
// Created by ascadian on 12/5/17.
//

#include <QMainWindow>
#include "streamwidget.h"
#include "ui_streamwidget.h"
#include "../Widgets/qtopencvwidget.h"
#include "../Widgets/pointswidget.h"

StreamWidget::StreamWidget(QWidget *parent) :
        QDockWidget(parent),
        ui(new Ui::StreamWidget)
{
    ui->setupUi(this);

    m_frameTimer.start();
}

StreamWidget::~StreamWidget() {
    detachObservers();
    delete ui;
}


void StreamWidget::detachObservers() {
    if(m_observer.IsValid()) {
        m_observer.Detach();
    }
    if(m_observer2.IsValid()) {
        m_observer2.Detach();
    }
}


// TODO: display size stats in the status line
void StreamWidget::updateStats() {
    m_frameTimer.update();
    ui->statsLabel->setText(QString(m_frameTimer.formatStats().str().c_str()));
}


void StreamWidget::setEvent(EventsT<cv::Mat> &event, StreamTypes type) {
    auto name = Stream::StreamTypesToString(type);
    setWindowTitle(QString::fromStdString(name));

    detachObservers();
    delete visWidget;

    QTOpenCVWidget *widget = new QTOpenCVWidget(this);
    visWidget = widget;
    visWidget->setFocusPolicy(Qt::ClickFocus);
    ui->showLayout->addWidget(visWidget);

    switch (type) {
        case StreamTypes::COLOR:
        case StreamTypes::REGISTERED_COLOR:
        {
            m_observer = Observe(event, [this, event, widget](const cv::Mat image) {
                widget->showImage(image);
                updateStats();
            });
            D::SetNodeName(*m_observer.nodePtr_, "StreamWidget::m_observer");
            break;
        }

        case StreamTypes::IR:
        case StreamTypes::DEPTH:
        case StreamTypes::UNDISTORTED_DEPTH:
        {
            auto stretched = Transform(event, [](const cv::Mat image) {
                // Histogram Equalization
                double min;
                double max;
                cv::minMaxIdx(image, &min, &max);
                cv::Mat adjMap = cv::Mat();
                // expand your range to 0..255. Similar to histEq();
                //(*image).convertTo(adjMap, CV_8UC1, 255 / (max-min), -255*min/(max-min)); // try this if the other method fails
                image.convertTo(adjMap, CV_8UC1, 255 / (max - min), -min);
                return adjMap;
            });
            D::SetNodeName(*stretched.ptr_, "StreamWidget::stretched");
            m_observer = Observe(stretched, [this, widget](const cv::Mat image) {
                widget->showImage(image);
                updateStats();
            });
            D::SetNodeName(*m_observer.nodePtr_, "StreamWidget::m_observer");
            break;
        }
        case StreamTypes::UNKNOWN:break;
        case StreamTypes::EXTRINSICS:break;
        case StreamTypes::INTRINSICS:break;
        case StreamTypes::COMPRESSED:break;
    }
}



void StreamWidget::setEventCloud(EventsT<cv::Mat> &pos) {
    setWindowTitle("3d Cloud");

    detachObservers();
    delete visWidget;

    PointsWidget *widget = new PointsWidget(this);
    visWidget = widget;
    widget->setFocusPolicy(Qt::ClickFocus);
    ui->showLayout->addWidget(widget);

    m_observer2 = Observe(pos, [this, widget](const cv::Mat &pos) {
        widget->setPointPos(pos);
        updateStats();
    });
    D::SetNodeName(*m_observer2.nodePtr_, "StreamWidget::m_observer2");
}

void StreamWidget::setEventCloud(EventsT<cv::Mat> &pos,
                            EventsT<cv::Mat> &color) {

    setEventCloud(pos);

    PointsWidget *widget = static_cast<PointsWidget*>(visWidget);
    m_observer = Observe(color, [this, widget](const cv::Mat &colors) {
        widget->setPointColors(colors);
        // don't update timer here, as we already do that in the cloud update
    });
    D::SetNodeName(*m_observer.nodePtr_, "StreamWidget::m_observer");
}
